@@ -222,7 +222,6 @@ class HandlerSocket(threading.local):
             :exc:`~.exceptions.OperationalError` otherwise.
         :rtype: :class:`~.Connection` instance
         """
-        conn = None
         connections = self.connections[:]
         random.shuffle(connections)
         # Try looking up for index_id in index_map - we should use same connections
@@ -286,7 +285,10 @@ class HandlerSocket(threading.local):
         columns = int(tokens[1])
         decoded_tokens = imap(decode, tokens[2:])
         # Divide response tokens list by number of columns
-        data = zip(*[decoded_tokens]*columns)
+        if columns > 1:
+            data = zip(*[decoded_tokens]*columns)
+        else:
+            data = list(decoded_tokens)
 
         return data
 
@@ -420,13 +422,14 @@ class ReadSocket(HandlerSocket):
 class WriteSocket(HandlerSocket):
     """HandlerSocket client for write operations."""
 
-    MODIFY_OPERATIONS = ('U', 'D')
+    MODIFY_OPERATIONS = ('U', 'D', '+', '-', 'U?', 'D?', '+?', '-?')
 
     def find_modify(self, index_id, operation, columns, modify_operation,
                     modify_columns=[], limit=0, offset=0):
         """Updates/deletes row(s) using opened index,
 
-        Returns number of modified rows.
+        Returns number of modified rows or a list of original values in case
+        ``modify_operation`` ends with ``?``.
 
         Raises ``ValueError`` if given data doesn't validate.
 
@@ -446,7 +449,7 @@ class WriteSocket(HandlerSocket):
             one row. In case multiple rows are expected to be changed, ``limit``
             must be set explicitly, HS wont change all found rows by default.
         :param integer offset: optional offset of rows to search for.
-        :rtype: int
+        :rtype: list
 
         """
         if operation not in self.FIND_OPERATIONS \
@@ -456,7 +459,8 @@ class WriteSocket(HandlerSocket):
         if not check_columns(columns):
             raise ValueError('Columns must be a non-empty iterable.')
 
-        if modify_operation == 'U' and not check_columns(modify_columns):
+        if modify_operation in ('U', '+', '-', 'U?', '+?', '-?') \
+            and not check_columns(modify_columns):
             raise ValueError('Modify_columns must be a non-empty iterable for update operation')
 
         query = chain(
@@ -468,7 +472,7 @@ class WriteSocket(HandlerSocket):
 
         response = self._call(index_id, query, force_index=True)
 
-        return int(response[0][0])
+        return response
 
     def insert(self, index_id, columns):
         """Inserts single row using opened index.
